// Generated by CoffeeScript 2.7.0
(function() {
  // ***** BEGIN LICENSE BLOCK *****
  // Copyright (c) 2011-2012 VMware, Inc.

  // For the license see COPYING.
  // ***** END LICENSE BLOCK *****
  var FayeWebsocket, RawWebsocketSessionReceiver, Transport, WebSocketReceiver, transport, utils;

  FayeWebsocket = require('faye-websocket');

  utils = require('./utils');

  transport = require('./transport');

  exports.app = {
    _websocket_check: function(req, connection, head) {
      if (!FayeWebsocket.isWebSocket(req)) {
        throw {
          status: 400,
          message: 'Not a valid websocket request'
        };
      }
    },
    sockjs_websocket: function(req, connection, head) {
      var ws;
      this._websocket_check(req, connection, head);
      ws = new FayeWebsocket(req, connection, head, null, this.options.faye_server_options);
      ws.onopen = () => {
        // websockets possess no session_id
        return transport.registerNoSession(req, this, new WebSocketReceiver(ws, connection));
      };
      return true;
    },
    raw_websocket: function(req, connection, head) {
      var ver, ws;
      this._websocket_check(req, connection, head);
      ver = req.headers['sec-websocket-version'] || '';
      if (['8', '13'].indexOf(ver) === -1) {
        throw {
          status: 400,
          message: 'Only supported WebSocket protocol is RFC 6455.'
        };
      }
      ws = new FayeWebsocket(req, connection, head, null, this.options.faye_server_options);
      ws.onopen = () => {
        return new RawWebsocketSessionReceiver(req, connection, this, ws);
      };
      return true;
    }
  };

  WebSocketReceiver = (function() {
    class WebSocketReceiver extends transport.GenericReceiver {
      constructor(ws, connection) {
        var x;
        try {
          connection.setKeepAlive(true, 5000);
          connection.setNoDelay(true);
        } catch (error) {
          x = error;
        }
        super(connection);
        this.ws = ws;
        // setUp() is called by super(), then ws was not initialized
        this.ws.addEventListener('close', this.thingy_end_cb);
        this.connection = connection;
        this.ws.addEventListener('message', (m) => {
          return this.didMessage(m.data);
        });
        this.heartbeat_cb = () => {
          return this.heartbeat_timeout();
        };
      }

      setUp() {
        super.setUp();
        if (this.ws) {
          return this.ws.addEventListener('close', this.thingy_end_cb);
        }
      }

      tearDown() {
        this.ws.removeEventListener('close', this.thingy_end_cb);
        return super.tearDown();
      }

      didMessage(payload) {
        var i, len, message, msg, results, x;
        if (this.ws && this.session && payload.length > 0) {
          try {
            message = JSON.parse(payload);
          } catch (error) {
            x = error;
            return this.didClose(3000, 'Broken framing.');
          }
          if (payload[0] === '[') {
            results = [];
            for (i = 0, len = message.length; i < len; i++) {
              msg = message[i];
              results.push(this.session.didMessage(msg));
            }
            return results;
          } else {
            return this.session.didMessage(message);
          }
        }
      }

      doSendFrame(payload) {
        var x;
        if (this.ws) {
          try {
            this.ws.send(payload);
            return true;
          } catch (error) {
            x = error;
          }
        }
        return false;
      }

      didClose(status = 1000, reason = "Normal closure") {
        var x;
        super.didClose();
        try {
          this.ws.close(status, reason, false);
        } catch (error) {
          x = error;
        }
        this.ws = null;
        return this.connection = null;
      }

      heartbeat() {
        var hto_ref, supportsHeartbeats;
        supportsHeartbeats = this.ws.ping(null, function() {
          return clearTimeout(hto_ref);
        });
        if (supportsHeartbeats) {
          return hto_ref = setTimeout(this.heartbeat_cb, 10000);
        } else {
          return super.heartbeat();
        }
      }

      heartbeat_timeout() {
        if (this.session != null) {
          return this.session.close(3000, 'No response from heartbeat');
        }
      }

    };

    WebSocketReceiver.prototype.protocol = "websocket";

    return WebSocketReceiver;

  }).call(this);

  Transport = transport.Transport;

  // Inheritance only for decorateConnection.
  RawWebsocketSessionReceiver = class RawWebsocketSessionReceiver extends transport.Session {
    constructor(req, conn, server, ws) {
      super();
      this.ws = ws;
      this.prefix = server.options.prefix;
      this.readyState = Transport.OPEN;
      this.recv = {
        connection: conn,
        protocol: "websocket-raw"
      };
      this.connection = new transport.SockJSConnection(this);
      this.decorateConnection(req);
      server.emit('connection', this.connection);
      this._end_cb = () => {
        return this.didClose();
      };
      this.ws.addEventListener('close', this._end_cb);
      this._message_cb = (m) => {
        return this.didMessage(m);
      };
      this.ws.addEventListener('message', this._message_cb);
    }

    didMessage(m) {
      if (this.readyState === Transport.OPEN) {
        this.connection.emit('data', m.data);
      }
    }

    send(payload) {
      if (this.readyState !== Transport.OPEN) {
        return false;
      }
      this.ws.send(payload);
      return true;
    }

    close(status = 1000, reason = "Normal closure") {
      if (this.readyState !== Transport.OPEN) {
        return false;
      }
      this.readyState = Transport.CLOSING;
      this.ws.close(status, reason, false);
      return true;
    }

    didClose() {
      var x;
      if (!this.ws) {
        return;
      }
      this.ws.removeEventListener('message', this._message_cb);
      this.ws.removeEventListener('close', this._end_cb);
      try {
        this.ws.close(1000, "Normal closure", false);
      } catch (error) {
        x = error;
      }
      this.ws = null;
      this.readyState = Transport.CLOSED;
      this.connection.emit('end');
      this.connection.emit('close');
      return this.connection = null;
    }

  };

}).call(this);
