// Generated by CoffeeScript 2.7.0
(function() {
  // ***** BEGIN LICENSE BLOCK *****
  // Copyright (c) 2011-2012 VMware, Inc.

  // For the license see COPYING.
  // ***** END LICENSE BLOCK *****
  var GenericReceiver, MAP, ResponseReceiver, Session, SockJSConnection, Transport, closeFrame, register, stream, utils, uuidv4;

  stream = require('stream');

  ({
    v4: uuidv4
  } = require('uuid'));

  utils = require('./utils');

  Transport = class Transport {};

  Transport.CONNECTING = 0;

  Transport.OPEN = 1;

  Transport.CLOSING = 2;

  Transport.CLOSED = 3;

  closeFrame = function(status, reason) {
    return 'c' + JSON.stringify([status, reason]);
  };

  SockJSConnection = class SockJSConnection extends stream.Stream {
    constructor(_session) {
      super();
      this._session = _session;
      this.id = uuidv4();
      this.headers = {};
      this.prefix = this._session.prefix;
    }

    toString() {
      return '<SockJSConnection ' + this.id + '>';
    }

    write(string) {
      return this._session.send('' + string);
    }

    end(string) {
      if (string) {
        this.write(string);
      }
      this.close();
      return null;
    }

    close(code, reason) {
      return this._session.close(code, reason);
    }

    destroy() {
      this.end();
      return this.removeAllListeners();
    }

    destroySoon() {
      return this.destroy();
    }

  };

  SockJSConnection.prototype.__defineGetter__('readable', function() {
    return this._session.readyState === Transport.OPEN;
  });

  SockJSConnection.prototype.__defineGetter__('writable', function() {
    return this._session.readyState === Transport.OPEN;
  });

  SockJSConnection.prototype.__defineGetter__('readyState', function() {
    return this._session.readyState;
  });

  MAP = {};

  Session = class Session {
    constructor(session_id, server) {
      this.session_id = session_id;
      this.heartbeat_delay = server.options.heartbeat_delay;
      this.disconnect_delay = server.options.disconnect_delay;
      this.prefix = server.options.prefix;
      this.send_buffer = [];
      this.is_closing = false;
      this.readyState = Transport.CONNECTING;
      if (this.session_id) {
        MAP[this.session_id] = this;
      }
      this.timeout_cb = () => {
        return this.didTimeout();
      };
      this.to_tref = setTimeout(this.timeout_cb, this.disconnect_delay);
      this.connection = new SockJSConnection(this);
      this.emit_open = () => {
        this.emit_open = null;
        return server.emit('connection', this.connection);
      };
    }

    register(req, recv) {
      if (this.recv) {
        recv.doSendFrame(closeFrame(2010, "Another connection still open"));
        recv.didClose();
        return;
      }
      if (this.to_tref) {
        clearTimeout(this.to_tref);
        this.to_tref = null;
      }
      if (this.readyState === Transport.CLOSING) {
        this.flushToRecv(recv);
        recv.doSendFrame(this.close_frame);
        recv.didClose();
        this.to_tref = setTimeout(this.timeout_cb, this.disconnect_delay);
        return;
      }
      // Registering. From now on 'unregister' is responsible for
      // setting the timer.
      this.recv = recv;
      this.recv.session = this;
      // Save parameters from request
      this.decorateConnection(req);
      // first, send the open frame
      if (this.readyState === Transport.CONNECTING) {
        this.recv.doSendFrame('o');
        this.readyState = Transport.OPEN;
        // Emit the open event, but not right now
        process.nextTick(this.emit_open);
      }
      // At this point the transport might have gotten away (jsonp).
      if (!this.recv) {
        return;
      }
      this.tryFlush();
    }

    decorateConnection(req) {
      var address, headers, i, key, len, ref, remoteAddress, remotePort, socket, x;
      // Store the last known address.
      if (!(socket = this.recv.connection)) {
        socket = this.recv.response.connection;
      }
      try {
        remoteAddress = socket.remoteAddress;
        remotePort = socket.remotePort;
        address = socket.address();
      } catch (error) {
        x = error;
      }
      if (remoteAddress) {
        // All-or-nothing
        this.connection.remoteAddress = remoteAddress;
        this.connection.remotePort = remotePort;
        this.connection.address = address;
      }
      this.connection.url = req.url;
      this.connection.pathname = req.pathname;
      this.connection.protocol = this.recv.protocol;
      headers = {};
      ref = ['referer', 'x-client-ip', 'x-forwarded-for', 'x-forwarded-host', 'x-forwarded-port', 'x-cluster-client-ip', 'via', 'x-real-ip', 'x-forwarded-proto', 'x-ssl', 'dnt', 'host', 'user-agent', 'accept-language'];
      for (i = 0, len = ref.length; i < len; i++) {
        key = ref[i];
        if (req.headers[key]) {
          headers[key] = req.headers[key];
        }
      }
      if (headers) {
        return this.connection.headers = headers;
      }
    }

    unregister() {
      var delay;
      delay = this.recv.delay_disconnect;
      this.recv.session = null;
      this.recv = null;
      if (this.to_tref) {
        clearTimeout(this.to_tref);
      }
      if (delay) {
        return this.to_tref = setTimeout(this.timeout_cb, this.disconnect_delay);
      } else {
        return this.timeout_cb();
      }
    }

    flushToRecv(recv) {
      var sb;
      if (this.send_buffer.length > 0) {
        [sb, this.send_buffer] = [this.send_buffer, []];
        recv.doSendBulk(sb);
        return true;
      }
      return false;
    }

    tryFlush() {
      var x;
      if (!this.flushToRecv(this.recv) || !this.to_tref) {
        if (this.to_tref) {
          clearTimeout(this.to_tref);
        }
        x = () => {
          if (this.recv) {
            this.to_tref = setTimeout(x, this.heartbeat_delay);
            return this.recv.heartbeat();
          }
        };
        this.to_tref = setTimeout(x, this.heartbeat_delay);
      }
    }

    didTimeout() {
      if (this.to_tref) {
        clearTimeout(this.to_tref);
        this.to_tref = null;
      }
      if (this.readyState !== Transport.CONNECTING && this.readyState !== Transport.OPEN && this.readyState !== Transport.CLOSING) {
        throw Error('INVALID_STATE_ERR');
      }
      if (this.recv) {
        throw Error('RECV_STILL_THERE');
      }
      this.readyState = Transport.CLOSED;
      // Node streaming API is broken. Reader defines 'close' and 'end'
      // but Writer defines only 'close'. 'End' isn't optional though.
      //   http://nodejs.org/docs/v0.5.8/api/streams.html#event_close_
      this.connection.emit('end');
      this.connection.emit('close');
      this.connection = null;
      if (this.session_id) {
        delete MAP[this.session_id];
        return this.session_id = null;
      }
    }

    didMessage(payload) {
      if (this.readyState === Transport.OPEN) {
        this.connection.emit('data', payload);
      }
    }

    send(payload) {
      if (this.readyState !== Transport.OPEN) {
        return false;
      }
      this.send_buffer.push('' + payload);
      if (this.recv) {
        this.tryFlush();
      }
      return true;
    }

    close(status = 1000, reason = "Normal closure") {
      if (this.readyState !== Transport.OPEN) {
        return false;
      }
      this.readyState = Transport.CLOSING;
      this.close_frame = closeFrame(status, reason);
      if (this.recv) {
        // Go away. doSendFrame can trigger didClose which can
        // trigger unregister. Make sure the @recv is not null.
        this.recv.doSendFrame(this.close_frame);
        if (this.recv) {
          this.recv.didClose();
        }
        if (this.recv) {
          this.unregister();
        }
      }
      return true;
    }

  };

  Session.bySessionId = function(session_id) {
    if (!session_id) {
      return null;
    }
    return MAP[session_id] || null;
  };

  register = function(req, server, session_id, receiver) {
    var session;
    session = Session.bySessionId(session_id);
    if (!session) {
      session = new Session(session_id, server);
    }
    session.register(req, receiver);
    return session;
  };

  exports.register = function(req, server, receiver) {
    return register(req, server, req.session, receiver);
  };

  exports.registerNoSession = function(req, server, receiver) {
    return register(req, server, void 0, receiver);
  };

  GenericReceiver = class GenericReceiver {
    constructor(thingy) {
      this.thingy = thingy;
      this.setUp(this.thingy);
    }

    setUp() {
      this.thingy_end_cb = () => {
        return this.didAbort();
      };
      this.thingy.addListener('close', this.thingy_end_cb);
      return this.thingy.addListener('end', this.thingy_end_cb);
    }

    tearDown() {
      this.thingy.removeListener('close', this.thingy_end_cb);
      this.thingy.removeListener('end', this.thingy_end_cb);
      return this.thingy_end_cb = null;
    }

    didAbort() {
      this.delay_disconnect = false;
      return this.didClose();
    }

    didClose() {
      if (this.thingy) {
        this.tearDown(this.thingy);
        this.thingy = null;
      }
      if (this.session) {
        return this.session.unregister();
      }
    }

    doSendBulk(messages) {
      var m, q_msgs;
      q_msgs = (function() {
        var i, len, results;
        results = [];
        for (i = 0, len = messages.length; i < len; i++) {
          m = messages[i];
          results.push(utils.quote(m));
        }
        return results;
      })();
      return this.doSendFrame('a' + '[' + q_msgs.join(',') + ']');
    }

    heartbeat() {
      return this.doSendFrame('h');
    }

  };

  ResponseReceiver = (function() {
    // Write stuff to response, using chunked encoding if possible.
    class ResponseReceiver extends GenericReceiver {
      constructor(request, response, options) {
        var x;
        try {
          request.connection.setKeepAlive(true, 5000);
        } catch (error) {
          x = error;
        }
        super(request.connection);
        this.request = request;
        this.response = response;
        this.options = options;
        this.curr_response_size = 0;
        if (this.max_response_size === void 0) {
          this.max_response_size = this.options.response_limit;
        }
      }

      doSendFrame(payload) {
        var r, x;
        this.curr_response_size += payload.length;
        r = false;
        try {
          this.response.write(payload);
          r = true;
        } catch (error) {
          x = error;
        }
        if (this.max_response_size && this.curr_response_size >= this.max_response_size) {
          this.didClose();
        }
        return r;
      }

      didClose() {
        var x;
        super.didClose();
        try {
          this.response.end();
        } catch (error) {
          x = error;
        }
        return this.response = null;
      }

    };

    ResponseReceiver.prototype.max_response_size = void 0;

    ResponseReceiver.prototype.delay_disconnect = true;

    return ResponseReceiver;

  }).call(this);

  exports.GenericReceiver = GenericReceiver;

  exports.Transport = Transport;

  exports.Session = Session;

  exports.ResponseReceiver = ResponseReceiver;

  exports.SockJSConnection = SockJSConnection;

}).call(this);
